package app;

import controllers.GameOfLifeController;
import executors.CommandExecutor;
import executors.GameOfLifeSimulator;
import models.GameOfLife;
import mvc.Controller;
import mvc.View;
import views.Console;
import views.Graphical;

public class Application {

	public static void main(String[] args) {

		GameOfLife game = new GameOfLife();
		
		boolean guiMode = (args.length > 0 && args[0].equals("--gui"));
		
		// asynchronous executor for GUI mode
		CommandExecutor.initialize(guiMode);
		
		View view = null;
		if (guiMode) {
			// simulator executor required for GUI mode -- timer interval
			GameOfLifeSimulator.initialize(game, 1);
			view = new Graphical();
		} else {
			view = new Console();
		}
		
		// create the controller that ties everything together
		Controller controller = new GameOfLifeController(game, view);
		
		// allow the view to parse requests directly from the model and controller
		game.addObserver(view);
		controller.addObserver(view);
		
		view.display();
	}
}
