package executors;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import commands.Command;

public class CommandExecutor {

	private static CommandExecutor singleton;

	private ExecutorService executorService = null;

	/**
	 * retrieves the singleton instance of CommandExecutor
	 * 
	 * @return the instance of the singleton
	 * @throws IllegalStateException if CommandExecutor.initialize was never called
	 */
	public static CommandExecutor getInstance() {
		if (singleton == null) {
			throw new IllegalStateException("Never initialized CommandExecutor");
		}
		return singleton;
	}

	/**
	 * Default-initialize the CommandExecutor instance to be asynchronous
	 */
	public static void initialize() {
		initialize(true);
	}

	/**
	 * Initialize the CommandExecutor instance to some async status
	 * 
	 * @param async true if the instance should be asynchronous in nature
	 *              (GUI-friendly) or false if synchronous
	 */
	public static void initialize(boolean async) {
		singleton = new CommandExecutor(async);
	}

	private CommandExecutor(boolean async) {
		if (async) {
			this.executorService = Executors.newSingleThreadExecutor();
		} else {
			this.executorService = null;
		}
	}

	/**
	 * Executes a command to the service
	 * @param cmd command to execute
	 */
	public void submit(Command cmd) {
		if (executorService != null) {
			executorService.execute(cmd::execute);
		} else {
			cmd.execute();
		}
	}

}
