package lib;

import java.util.ArrayList;

/**
 * Observable object for the Observer design pattern
 * @author wkillian
 *
 */
public class Observable {
	
	private ArrayList<Observer> observers;
	
	public Observable() {
		observers = new ArrayList<Observer>();
	}

	/**
	 * Adds an Observer to the list of current Observers
	 * @param o an observer
	 */
	public void addObserver(Observer o) {
		observers.add(o);
	}
	
	/**
	 * Notifies all registered Observers about a tagged event with some data
	 * @param tag String used to identify the event
	 * @param data gemeroc data associated with the event
	 */
	protected void signal(String tag, Object data) {
		for (Observer o : observers) {
			o.observableUpdated(this, tag, data);
		}
	}
}
