package mvc;

import lib.Observable;
import lib.Observer;

/**
 * Controller component of MVC. Acts as the event handler. It is both an Observer and Observable.
 * @author wkillian
 *
 */
public abstract class Controller extends Observable implements Observer {
	
	protected View mView;
	protected Model mModel;
	
	/**
	 * Initializes the state of the controller by having the model and view be Observed by this instance
	 * @param model
	 * @param view
	 */
	protected Controller(Model model, View view) {
		this.mModel = model;
		this.mView = view;
		this.mModel.addObserver(this);
		this.mView.addObserver(this);
	}
	
	protected View getView() {
		return mView;
	}
	
	protected Model getModel() {
		return mModel;
	}
	
	public final void observableUpdated(Observable src, String tag, Object eventData) {
		if (src == getView()) {
			this.viewChange(tag, eventData);
		} else if (src == getModel()) {
			this.modelChange(tag, eventData);
		} else {
			throw new RuntimeException("Unknown signaler");
		}
	}

	/**
	 * Callback for when the view has been changed
	 * @param tag String for event tag
	 * @param data associated data from event
	 */
	protected abstract void viewChange(String tag, Object data);
	
	/**
	 * Callback for when the model has been changed
	 * @param tag String for event tag
	 * @param data associated data from event
	 */
	protected abstract void modelChange(String tag, Object data);
}
