package views;

import java.util.Scanner;

import lib.Observable;
import models.GameOfLife;
import mvc.View;

public class Console extends View {

	/**
	 * Prints the matrix
	 * @param newValue
	 */
	private void printWorld(boolean[][] newValue) {
		System.out.println();
		for (boolean[] row : newValue) {
			for (boolean cell : row) {
				System.out.print(cell ? '#' : '-');
				System.out.print(' ');
			}
			System.out.println();
		}
		System.out.println();
	}

	/**
	 * REPL interface
	 */
	@Override
	public void display() {
		Scanner input = new Scanner(System.in);
		try {
			while (true) {
				System.out.print("> ");
				String line = input.nextLine();
				try {
					String[] args = line.split("[ \t]{1,}");
					switch (args[0].toLowerCase()) {
					case "reset":
					case "initialize":
					case "step":
						signal("sim_command", args[0].toLowerCase());
						break;
					case "config":
						signal("state", "config");
						break;
					case "live":
					case "birth": {
						int low = Integer.parseInt(args[1]);
						int high = Integer.parseInt(args[2]);
						if (low < 1 || high < 1 || low > 9 || high > 9 || low > high) {
							throw new IllegalStateException("[low,high] must be in range [1,9] and low <= high");
						}
						signal(args[0].toLowerCase() + "Low", low);
						signal(args[0].toLowerCase() + "High", high);
						break;
					}
					case "seed": {
						long value = Long.parseLong(args[1]);
						signal("seed", value);
						break;
					}
					case "quit": {
						System.exit(0);
					}
					case "height":
					case "width": {
						int value = Integer.parseInt(args[1]);
						signal(args[0].toLowerCase(), value);
						break;
					}
					default:
						System.out.println("Unknown command: " + args[0] + '\n');
					case "help":
						System.out.println("Available Commands:");
						System.out.println("  reset");
						System.out.println("  initialize");
						System.out.println("  step");
						System.out.println("  birth {low} {high}");
						System.out.println("  live {low} {high}");
						System.out.println("  seed {num}");
						System.out.println("  width {dim}");
						System.out.println("  height {dim}");
						System.out.println("  config");
						System.out.println("  quit");
						break;
					}
				} catch (Exception e) {
					System.err.println("Invalid: " + e.getMessage());
				}
			}
		} catch (Exception e) {
			input.close();
		}
	}

	@Override
	public void observableUpdated(Observable src, String tag, Object eventData) {
		if (tag.equals("world")) {
			printWorld((boolean[][]) eventData);
		} else if (tag.equals("state")) {
			GameOfLife game = (GameOfLife)eventData;
			System.out.println();
			System.out.println("Current Configuration:");
			System.out.println("  birth " + game.getBirthLow() + " " + game.getBirthHigh());
			System.out.println("  live " + game.getLiveLow() + " " + game.getLiveHigh());
			System.out.println("  seed " + game.getSeed());
			System.out.println("  width " + game.getWidth());
			System.out.println("  height " + game.getHeight());
			System.out.println();
		}
	}

}
