package views;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;

import executors.GameOfLifeSimulator;
import lib.Observable;
import models.GameOfLife;
import mvc.View;
import views.components.RangeSlider;

public class Graphical extends View {

	private JFrame frame;
	private World world;
	private JPanel settings;
	private JPanel simControl;
	private JPanel ui;

	public Graphical() {
		ui = new JPanel();
		ui.setLayout(new BorderLayout());
		settings = makeSettingsPanel();
		simControl = makeSimuationControl();
		world = new World();
		ui.add(simControl, BorderLayout.NORTH);
		ui.add(world, BorderLayout.CENTER);
		ui.add(settings, BorderLayout.SOUTH);
	}

	private void addSlider(JPanel parent, String labelName, String tag, int currLow, int currHigh, int low, int high) {
		RangeSlider slider = new RangeSlider(low, high);
		JPanel group = new JPanel();
		group.add(new JLabel(labelName));
		group.add(slider);
		parent.add(group);
		slider.addChangeListener((l) -> {
			signal(tag + "Low", slider.getValue());
			signal(tag + "High", slider.getUpperValue());
		});
		slider.setValue(currLow);
		slider.setUpperValue(currHigh);
	}

	private <E extends Comparable<E>> void addBoundedNumberChooser(JPanel parent, String label, String tag,
			Number value, E low, E high) {
		JSpinner spinner = new JSpinner(new SpinnerNumberModel(value, low, high, 1));
		((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().setColumns(13);
		JPanel group = new JPanel();
		group.add(new JLabel(label));
		group.add(spinner);
		parent.add(group);
		spinner.addChangeListener((l) -> {
			signal(tag, spinner.getValue());
		});
	}

	private JPanel makeSettingsPanel() {
		JPanel panel = new JPanel();
		GridLayout layout = new GridLayout(3, 2);
		panel.setLayout(layout);
		addSlider(panel, "Birth Range", "birth", GameOfLife.DEFAULT_BIRTH_LOW, GameOfLife.DEFAULT_BIRTH_HIGH, 1, 9);
		addBoundedNumberChooser(panel, "Width", "width", GameOfLife.DEFAULT_WIDTH, 3, 100);
		addSlider(panel, "Live Range", "live", GameOfLife.DEFAULT_LIVE_LOW, GameOfLife.DEFAULT_LIVE_HIGH, 1, 9);
		addBoundedNumberChooser(panel, "Height", "height", GameOfLife.DEFAULT_HEIGHT, 3, 100);
		addBoundedNumberChooser(panel, "Seed", "seed", GameOfLife.DEFAULT_SEED, 0L, Long.MAX_VALUE);
		return panel;
	}

	public JPanel makeSimuationControl() {
		JButton startPause = new JButton("Start");
		JButton reset = new JButton("Reset");
		JSlider stepSpeed = new JSlider(1, 60, 1);
		startPause.setEnabled(true);
		reset.setEnabled(true);

		startPause.addActionListener((l) -> {
			if (startPause.getText() == "Pause") {
				startPause.setText("Start");
				GameOfLifeSimulator.getInstance().pause();
			} else {
				startPause.setText("Pause");
				GameOfLifeSimulator.getInstance().start();
			}
		});
		reset.addActionListener((l) -> {
			GameOfLifeSimulator.getInstance().pause();
			GameOfLifeSimulator.getInstance().reset();
			startPause.setText("Start");
		});

		stepSpeed.addChangeListener((l) -> {
			int speed = stepSpeed.getValue();
			GameOfLifeSimulator.getInstance().setStepAmount(speed);
		});

		JPanel panel = new JPanel();
		panel.add(reset);
		panel.add(startPause);
		panel.add(stepSpeed);
		return panel;
	}

	@Override
	public void display() {
		frame = new JFrame("Game of Life");
		frame.getContentPane().add(ui);
		frame.setPreferredSize(new Dimension(800, 600));
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		frame.setMinimumSize(new Dimension(600, 400));
	}

	@Override
	public void observableUpdated(Observable src, String tag, Object eventData) {
		if (tag.equals("world")) {
			world.setData((boolean[][]) eventData);
			world.repaint();
		}
	}
}
